(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/common'), require('@angular/core'), require('@angular/forms')) :
    typeof define === 'function' && define.amd ? define('nthm', ['exports', '@angular/common', '@angular/core', '@angular/forms'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.nthm = {}, global.ng.common, global.ng.core, global.ng.forms));
}(this, (function (exports, common, core, forms) { 'use strict';

    /**
     * The ToggleLabel directive allows you to customize the label for the "On" and "Off" states,
     * allowing for more robust and complex displays.
     * This directive must be used in conjunction with a ng-template.
     */
    var NgToggleLabel = /** @class */ (function () {
        function NgToggleLabel(templateRef, elRef) {
            this.templateRef = templateRef;
            this.elRef = elRef;
        }
        Object.defineProperty(NgToggleLabel.prototype, "element", {
            get: function () {
                return this.elRef;
            },
            enumerable: false,
            configurable: true
        });
        return NgToggleLabel;
    }());
    NgToggleLabel.decorators = [
        { type: core.Directive, args: [{ selector: 'ng-template[ngToggleLabel]' },] }
    ];
    NgToggleLabel.ctorParameters = function () { return [
        { type: core.TemplateRef },
        { type: core.ElementRef }
    ]; };
    NgToggleLabel.propDecorators = {
        forLabel: [{ type: core.Input }]
    };

    /**
     * The Toggle directive allows for standalone or checkbox-enabled switch toggling via a UI element.
     * The toggle is styled using Bootstrap v4+ classes.
     * Accessibility implemented according to
     * https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/Roles/checkbox_role
     */
    var NgToggle = /** @class */ (function () {
        function NgToggle(ngZone, element) {
            this.ngZone = ngZone;
            this.element = element;
            /**
             * Display text when toggled in the "On" position
             */
            this.onText = 'On';
            /**
             * Display text when toggled in the "Off" position
             */
            this.offText = 'Off';
            /**
             * Bootstrap color scheme when toggled in the "On" position (i.e. "primary" translates to "btn-primary")
             */
            this.onColor = 'primary';
            /**
             * Bootstrap color scheme when toggled in the "Off" position (i.e. "primary" translates to "btn-primary")
             */
            this.offColor = 'secondary';
            /**
             * Button size to display the toggle
             */
            this.size = '';
            /**
             * Whether the toggle is disabled or not
             */
            this.disabled = false;
            /**
             * An event fired when the user causes a change.
             * The payload of the event is the currently selected value.
             */
            this.valueChange = new core.EventEmitter();
            this.btnClass = true;
            this.tabindex = 0;
            this.role = 'checkbox';
            this.width = 0;
            this.handleWidth = 0;
            this._animate = true;
            this._innerAnimate = true;
            this._innerState = false;
            this._innerWidth = 'auto';
            this._disableInitialAnimation = false;
            this._dragStart = null;
            this._dragEnd = null;
            this._initialized = false;
            this._hidden = false;
        }
        Object.defineProperty(NgToggle.prototype, "outerAnimate", {
            /**
             * Optional. Enable/Disable the initial transition animation. May prevent unwanted animation display.
             */
            set: function (value) {
                this._disableInitialAnimation = value;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "value", {
            get: function () {
                return this._innerState;
            },
            set: function (value) {
                this.setState(value);
            },
            enumerable: false,
            configurable: true
        });
        NgToggle.prototype.ngAfterViewInit = function () {
            var _this = this;
            this.calculateWidth();
            this._initialized = true;
            this.ngZone.run(function () { return setTimeout(function () { return _this._disableInitialAnimation = false; }); });
        };
        NgToggle.prototype.ngAfterViewChecked = function () {
            var hidden = this.element.nativeElement.offsetParent === null;
            if ((!this._initialized || this.width === 0) && this._hidden !== hidden) {
                this._initialized = false;
                this.calculateWidth();
                this._initialized = true;
            }
            this._hidden = hidden;
        };
        NgToggle.prototype.ngAfterContentInit = function () {
            var onElement = this.labelElements.find(function (item) { return item.forLabel.toLowerCase() === 'on'; });
            var offElement = this.labelElements.find(function (item) { return item.forLabel.toLowerCase() === 'off'; });
            if (onElement) {
                this.onLabel = onElement;
            }
            if (offElement) {
                this.offLabel = offElement;
            }
            if (onElement || offElement) {
                this.calculateWidth(true);
            }
        };
        NgToggle.prototype.ngOnChanges = function (changes) {
            if ('onText' in changes || 'offText' in changes || 'size' in changes) {
                this._initialized = false;
                this.calculateWidth(this._initialized);
                this._initialized = true;
            }
        };
        Object.defineProperty(NgToggle.prototype, "largeButton", {
            get: function () {
                return this.size === 'lg';
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "smallButton", {
            get: function () {
                return this.size === 'sm';
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "handleDark", {
            get: function () {
                return (this.value && this.onColor === 'light') || (!this.value && this.offColor === 'light');
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "indeterminate", {
            get: function () {
                return this._innerState === null || typeof this._innerState === 'undefined';
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "toggledOn", {
            get: function () {
                return this.innerState === true;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "toggledOff", {
            get: function () {
                return this.innerState === false;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "innerState", {
            get: function () {
                return this._innerState;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "ariaCheckedValue", {
            get: function () {
                if (this.indeterminate) {
                    return 'mixed';
                }
                return this.value ? 'true' : 'false';
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "animate", {
            get: function () {
                return this._animate && (!this._disableInitialAnimation || this._initialized && !this._disableInitialAnimation);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "marginLeft", {
            get: function () {
                var margin = 0;
                if (!this._initialized) {
                }
                else if (this.indeterminate || this._innerState === null || typeof this._innerState === 'undefined') {
                    margin = -(this.width / 2);
                }
                else if (this._dragEnd) {
                    margin = this._dragEnd;
                }
                else if (!this._innerState) {
                    margin = -this.width;
                }
                return margin + 'px';
            },
            enumerable: false,
            configurable: true
        });
        NgToggle.prototype.handleResize = function () {
            this._initialized = false;
            this.calculateWidth(true);
            this._initialized = true;
        };
        NgToggle.prototype.handleClick = function () {
            if (!this.disabled && !this._dragEnd) {
                this.setState(!this._innerState);
            }
            else if (this._dragEnd) {
                this._dragEnd = null;
            }
        };
        NgToggle.prototype.onTouchStart = function (event) {
            this.onDragStart(event);
        };
        NgToggle.prototype.onMouseDown = function (event) {
            this.onDragStart(event);
        };
        NgToggle.prototype.onTouchMove = function (event) {
            this.onDragMove(event);
        };
        NgToggle.prototype.onMouseMove = function (event) {
            this.onDragMove(event);
        };
        NgToggle.prototype.onTouchEnd = function (event) {
            this.onDragEnd(event, true);
        };
        NgToggle.prototype.onMouseUp = function (event) {
            this.onDragEnd(event);
        };
        NgToggle.prototype.onMouseLeave = function (event) {
            this.onDragEnd(event, true);
        };
        NgToggle.prototype.onKeyDown = function (event) {
            if (!event.key || this.disabled) {
                return;
            }
            switch (event.key) {
                case 'Left':
                case 'ArrowLeft':
                    event.preventDefault();
                    event.stopImmediatePropagation();
                    this.setState(false);
                    break;
                case 'Right':
                case 'ArrowRight':
                    event.preventDefault();
                    event.stopImmediatePropagation();
                    this.setState(true);
                    break;
                case 'Spacebar':
                case ' ':
                    event.preventDefault();
                    event.stopImmediatePropagation();
                    this.setState(!this.value);
            }
        };
        NgToggle.prototype.onDragStart = function (event) {
            if (event.target === this.handle$) {
                if (this._dragStart || this.disabled) {
                    return;
                }
                event.preventDefault();
                event.stopPropagation();
                this._dragStart = (event.pageX || event.touches[0].pageX) - parseInt(this.container$.style.marginLeft, 10);
                if (this._animate) {
                    this._animate = !this._animate;
                }
            }
        };
        NgToggle.prototype.onDragMove = function (event) {
            if (this._dragStart) {
                event.preventDefault();
                var difference = (event.pageX || event.touches[0].pageX) - this._dragStart;
                if (difference < -(Number(this.width)) || difference > 0) {
                    return;
                }
                this._dragEnd = difference;
            }
        };
        NgToggle.prototype.onDragEnd = function (event, clearDragEnd) {
            if (clearDragEnd === void 0) { clearDragEnd = false; }
            if (this._dragStart) {
                event.preventDefault();
                event.stopPropagation();
                if (this._dragEnd) {
                    this.setState(this._dragEnd > -(Number(this.width) / 2));
                }
                this._dragStart = null;
                if (clearDragEnd) {
                    this._dragEnd = null;
                }
                if (this._innerAnimate && !this._animate) {
                    this._animate = true;
                }
            }
        };
        NgToggle.prototype.calculateWidth = function (disableAnimation) {
            var _this = this;
            if (disableAnimation === void 0) { disableAnimation = false; }
            if (disableAnimation && this._innerAnimate) {
                this._animate = false;
            }
            var initialized = this._initialized;
            if (!initialized) {
                this.container$.style.width = 'auto';
                this.element$.style.width = 'auto';
            }
            this.on$.style.width = 'auto';
            this.off$.style.width = 'auto';
            setTimeout(function () {
                var width = _this._innerWidth;
                if (_this._innerWidth === 'auto') {
                    width = Math.max(_this.on$.offsetWidth, _this.off$.offsetWidth);
                }
                _this.handleWidth = _this.handle$.offsetWidth;
                _this.width = Number(width);
                if (!initialized) {
                    _this.container$.style.width = ((_this.width * 2) + _this.handleWidth) + 'px';
                    _this.element$.style.width = (_this.width + _this.handleWidth) + 'px';
                }
                _this.ngZone.run(function () {
                    _this.on$.style.width = _this.width + 'px';
                    _this.off$.style.width = _this.width + 'px';
                    setTimeout(function () {
                        if (disableAnimation && _this._innerAnimate) {
                            _this._animate = true;
                        }
                    });
                });
            });
        };
        NgToggle.prototype.setState = function (value) {
            if (value !== this._innerState) {
                this._innerState = value;
                this.valueChange.emit(this._innerState);
            }
        };
        Object.defineProperty(NgToggle.prototype, "element$", {
            get: function () {
                return this.element.nativeElement;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "on$", {
            get: function () {
                return this.onElement.nativeElement;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "off$", {
            get: function () {
                return this.offElement.nativeElement;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "handle$", {
            get: function () {
                return this.handleElement.nativeElement;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgToggle.prototype, "container$", {
            get: function () {
                return this.containerElement.nativeElement;
            },
            enumerable: false,
            configurable: true
        });
        return NgToggle;
    }());
    NgToggle.decorators = [
        { type: core.Component, args: [{
                    selector: 'ng-toggle',
                    template: "\n    <span #container class=\"ng-toggle-container\" [style.marginLeft]=\"marginLeft\">\n      <span #on class=\"ng-toggle-on btn btn-{{onColor}}\" [class.btn-lg]=\"largeButton\"\n            [class.btn-sm]=\"smallButton\" [class.disabled]=\"disabled\">\n          <ng-template [ngTemplateOutlet]=\"onLabel?.templateRef\"></ng-template>\n          <ng-container *ngIf=\"!onLabel\">{{onText}}</ng-container>\n      </span>\n      <span #handle class=\"ng-toggle-handle btn\" [class.btn-lg]=\"largeButton\"\n            [class.btn-sm]=\"smallButton\" [class.disabled]=\"disabled\"\n            [class.btn-light]=\"!handleDark\" [class.btn-dark]=\"handleDark\">&nbsp;</span>\n      <span #off class=\"ng-toggle-off btn btn-{{offColor}}\" [class.btn-lg]=\"largeButton\"\n            [class.btn-sm]=\"smallButton\" [class.disabled]=\"disabled\">\n        <ng-template [ngTemplateOutlet]=\"offLabel?.templateRef\"></ng-template>\n        <ng-container *ngIf=\"!offLabel\">{{offText}}</ng-container>\n      </span>\n    </span>\n    <ng-content></ng-content>\n    ",
                    preserveWhitespaces: false,
                    encapsulation: core.ViewEncapsulation.None,
                    styles: ["ng-toggle{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;box-sizing:content-box;cursor:pointer;direction:ltr;display:inline-block;overflow:hidden;padding:0!important;position:relative;text-align:left;transition:border-color .15s ease-in-out,box-shadow .15s ease-in-out;user-select:none;vertical-align:middle;z-index:0}ng-toggle.disabled,ng-toggle.disabled .btn{cursor:default}ng-toggle input{height:1px;position:absolute;visibility:hidden;width:1px;z-index:-1}ng-toggle .ng-toggle-container,ng-toggle .ng-toggle-handle,ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{display:-ms-flexbox!important;display:flex!important}ng-toggle .ng-toggle-container{-ms-flex-align:stretch!important;-webkit-transform:translateZ(0);align-items:stretch!important;border-radius:0;top:0;transform:translateZ(0)}ng-toggle.ng-toggle-animate .ng-toggle-container{transition:margin-left .5s}ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{-ms-flex-align:center!important;align-items:center!important;border-radius:0;text-align:center;z-index:1}ng-toggle .ng-toggle-handle,ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;box-sizing:border-box;cursor:pointer;user-select:none}ng-toggle .ng-toggle-handle{-ms-flex-item-align:stretch!important;-ms-grid-row-align:stretch!important;align-self:stretch!important;margin-bottom:-1px;margin-top:-1px;padding-left:0;padding-right:0;text-align:center;width:1em;z-index:100}"]
                },] }
    ];
    NgToggle.ctorParameters = function () { return [
        { type: core.NgZone },
        { type: core.ElementRef }
    ]; };
    NgToggle.propDecorators = {
        onText: [{ type: core.Input }],
        offText: [{ type: core.Input }],
        onColor: [{ type: core.Input }],
        offColor: [{ type: core.Input }],
        size: [{ type: core.Input }],
        disabled: [{ type: core.Input }, { type: core.HostBinding, args: ['class.disabled',] }, { type: core.HostBinding, args: ['class.ng-toggle-disabled',] }],
        valueChange: [{ type: core.Output }],
        btnClass: [{ type: core.HostBinding, args: ['class.btn',] }],
        tabindex: [{ type: core.Input, args: ['tabindex',] }, { type: core.HostBinding, args: ['attr.tabindex',] }],
        role: [{ type: core.Input, args: ['role',] }, { type: core.HostBinding, args: ['attr.role',] }],
        containerElement: [{ type: core.ViewChild, args: ['container', { static: true },] }],
        onElement: [{ type: core.ViewChild, args: ['on', { static: true },] }],
        offElement: [{ type: core.ViewChild, args: ['off', { static: true },] }],
        handleElement: [{ type: core.ViewChild, args: ['handle', { static: true },] }],
        labelElements: [{ type: core.ContentChildren, args: [NgToggleLabel,] }],
        outerAnimate: [{ type: core.Input, args: ['disableInitialAnimation',] }],
        value: [{ type: core.Input }],
        largeButton: [{ type: core.HostBinding, args: ['class.btn-lg',] }, { type: core.HostBinding, args: ['class.ng-toggle-lg',] }],
        smallButton: [{ type: core.HostBinding, args: ['class.btn-sm',] }, { type: core.HostBinding, args: ['class.ng-toggle-sm',] }],
        indeterminate: [{ type: core.HostBinding, args: ['class.ng-toggle-indeterminate',] }],
        toggledOn: [{ type: core.HostBinding, args: ['class.ng-toggled-on',] }],
        toggledOff: [{ type: core.HostBinding, args: ['class.ng-toggled-off',] }],
        ariaCheckedValue: [{ type: core.HostBinding, args: ['attr.aria-checked',] }],
        animate: [{ type: core.HostBinding, args: ['class.ng-toggle-animate',] }],
        handleResize: [{ type: core.HostListener, args: ['window:resize',] }],
        handleClick: [{ type: core.HostListener, args: ['click',] }],
        onTouchStart: [{ type: core.HostListener, args: ['touchstart', ['$event'],] }],
        onMouseDown: [{ type: core.HostListener, args: ['mousedown', ['$event'],] }],
        onTouchMove: [{ type: core.HostListener, args: ['touchmove', ['$event'],] }],
        onMouseMove: [{ type: core.HostListener, args: ['mousemove', ['$event'],] }],
        onTouchEnd: [{ type: core.HostListener, args: ['touchend', ['$event'],] }],
        onMouseUp: [{ type: core.HostListener, args: ['mouseup', ['$event'],] }],
        onMouseLeave: [{ type: core.HostListener, args: ['mouseleave', ['$event'],] }],
        onKeyDown: [{ type: core.HostListener, args: ['keydown', ['$event'],] }]
    };

    var NgToggleAccessor = /** @class */ (function () {
        function NgToggleAccessor(_host) {
            var _this = this;
            this._host = _host;
            this._subscription = this._host.valueChange.subscribe(function (value) { return _this.onChange(value); });
        }
        NgToggleAccessor.prototype.ngOnDestroy = function () {
            if (this._subscription) {
                this._subscription.unsubscribe();
                this._subscription = null;
            }
        };
        NgToggleAccessor.prototype.onChange = function (_) {
            if (this._onChange) {
                this._onChange(this._host.value);
            }
        };
        NgToggleAccessor.prototype.onTouched = function () {
            if (this._onTouched) {
                this._onTouched();
            }
        };
        NgToggleAccessor.prototype.registerOnChange = function (fn) {
            this._onChange = fn;
        };
        NgToggleAccessor.prototype.registerOnTouched = function (fn) {
            this._onTouched = fn;
        };
        NgToggleAccessor.prototype.setDisabledState = function (isDisabled) {
            this._host.disabled = isDisabled;
        };
        NgToggleAccessor.prototype.writeValue = function (obj) {
            if (typeof obj === 'boolean' || obj === null) {
                this._host.value = obj;
            }
        };
        return NgToggleAccessor;
    }());
    NgToggleAccessor.decorators = [
        { type: core.Directive, args: [{
                    selector: 'ng-toggle',
                    host: { '(change)': 'onChange($event)', '(touch)': 'onTouched()' },
                    providers: [{ provide: forms.NG_VALUE_ACCESSOR, useExisting: core.forwardRef(function () { return NgToggleAccessor; }), multi: true }]
                },] }
    ];
    NgToggleAccessor.ctorParameters = function () { return [
        { type: NgToggle }
    ]; };

    var declarations = [
        NgToggle,
        NgToggleAccessor,
        NgToggleLabel,
    ];
    var NgToggleModule = /** @class */ (function () {
        function NgToggleModule() {
        }
        return NgToggleModule;
    }());
    NgToggleModule.decorators = [
        { type: core.NgModule, args: [{ imports: [common.CommonModule], exports: declarations, declarations: declarations },] }
    ];

    /**
     * Generated bundle index. Do not edit.
     */

    exports.NgToggle = NgToggle;
    exports.NgToggleAccessor = NgToggleAccessor;
    exports.NgToggleLabel = NgToggleLabel;
    exports.NgToggleModule = NgToggleModule;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ng-toggle.umd.js.map
